#ifndef EPT_DEBTAGS_SOURCEDIR_TCC
#define EPT_DEBTAGS_SOURCEDIR_TCC

/** @file
 * @author Enrico Zini <enrico@enricozini.org>
 * Debtags data source directory access
 */
#include <ept/config.h>
#include <ept/debtags/maint/sourcedir.h>

#include <tagcoll/input/zlib.h>
#include <tagcoll/input/stdio.h>

#ifdef HAVE_WIBBLE_1
# define D_NAME(d) (*d)
#else
# define D_NAME(d) (d->d_name)
#endif

namespace ept {
namespace debtags {

template<typename OUT>
void SourceDir::readTags(OUT out)
{
#ifdef HAVE_WIBBLE_1
	if (!exists()) return;
#else
	if (!valid()) return;
#endif

	for (const_iterator d = begin(); d != end(); ++d)
	{
		FileType type = fileType(D_NAME(d));
		if (type == TAG)
		{
			// Read uncompressed data
			tagcoll::input::Stdio in(path() + "/" + D_NAME(d));

			// Read the collection
			tagcoll::textformat::parse(in, out);
		}
		else if (type == TAGGZ)
		{
			// Read compressed data
			tagcoll::input::Zlib in(path() + "/" + D_NAME(d));

			// Read the collection
			tagcoll::textformat::parse(in, out);
		}
	}
}

}
}

#include <tagcoll/TextFormat.tcc>

#endif

// -*- C++ -*-
// vim:set ts=4 sw=4:
